@@ -500,7 +500,10 @@ SEXP read_workbook(IntegerVector cols_in,
   
   bool has_strings = true;
   IntegerVector st_inds0 (1);
-  st_inds0[0] = string_inds[0];
+  
+  if (string_inds.size()) 
+    st_inds0[0] = string_inds[0];
+  
   if(is_true(all(is_na(st_inds0))))
     has_strings = false;
   
@@ -533,12 +536,11 @@ SEXP read_workbook(IntegerVector cols_in,
   CharacterVector col_names(nCols);
   IntegerVector removeFlag;
   int pos = 0;
-  
+
   // If we are told col_names exist take the first row and fill any gaps with X.i
   if(hasColNames){
     
     int row_1 = rows[0];
-    char name[6];
     
     IntegerVector row1_inds = which_cpp(rows == row_1);
     IntegerVector header_cols = cols[row1_inds];
@@ -547,24 +549,15 @@ SEXP read_workbook(IntegerVector cols_in,
     
     // looping over each column
     for(unsigned short i=0; i < nCols; i++){
-      
+      std::string nm = "X" + std::to_string(i + 1);
+
       if(missing_header[i]){  // a missing header element
-        
-	snprintf(name, 6, "X%hu", (unsigned short)(i+1));
-        // sprintf(&(name[0]), "X%u", i+1);
-        // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
-        // snprintf(&(name[0]), 10, "X%d", i+1);
-        col_names[i] = name;
-        
-      }else{  // this is a header elements 
-        
+        col_names[i] = nm.c_str();
+      }else{  // this is a header elements
+
         col_names[i] = v[pos];
         if(col_names[i] == "NA"){
-	  snprintf(name, 6, "X%hu", (unsigned short)(i+1));
-          // sprintf(&(name[0]), "X%du", i+1);
-          // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
-          // snprintf(&(name[0]), 10, "X%d", i+1);
-          col_names[i] = name;
+          col_names[i] = nm.c_str();
         }
         
         pos++;
@@ -618,12 +611,10 @@ SEXP read_workbook(IntegerVector cols_in,
     
     
   }else{ // else col_names is FALSE
-    char name[6];
-    for(int i =0; i < nCols; i++){
-	snprintf(name, 6, "X%hu", (unsigned short)(i+1));
-       // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
-      // sprintf(&(name[0]), "X%u", i+1);
-      col_names[i] = name;
+    for(unsigned short i =0; i < nCols; i++){
+
+      std::string nm = "X" + std::to_string(i + 1);
+      col_names[i] = nm.c_str();
     }
   }
   
@@ -637,7 +628,7 @@ SEXP read_workbook(IntegerVector cols_in,
   // Possible there are no string_inds to begin with and value of string_inds is 0
   // Possible we have string_inds but they have now all been used up by headers
   bool allNumeric = false;
-  if((string_inds.size() == 0) || (bool) all(is_na(string_inds)))
+  if((string_inds.size() == 0) || is_true(all(is_na(string_inds))))
     allNumeric = true;
   
   if(has_date){
@@ -645,7 +636,7 @@ SEXP read_workbook(IntegerVector cols_in,
       allNumeric = false;
   }
   
-  // If we have colnames some elements where used to create these -so we remove the corresponding number of elements
+  // If we have colnames some elements were used to create these -so we remove the corresponding number of elements
   if(hasColNames && has_date)
     is_date.erase(is_date.begin(), is_date.begin() + pos);
   
