@@ -2,6 +2,10 @@
 context("error without write permissions")
 
 test_that("test failed write errors for saveWorkbook", {
+  
+  skip_on_cran()
+  skip_on_ci()
+  
   temp_file <- tempfile()
   file.create(temp_file)
   Sys.chmod(path = temp_file, mode = "444")
